#========================================================================================
# state        private
#========================================================================================
package DE_EPAGES::AppStore::t::AppsClient;
use base DE_EPAGES::Database::API::Test;

use DE_EPAGES::AppStore::API::Object::AppsClient;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Status qw ( :constants :is status_message );
use Test::MockObject;
use Test::More;

use strict;

## no critic qw(ProhibitLongLines)

sub init {
    my $self = shift;

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {
        GUID => 'shopGUID'
    });

    $self->{'Client'} = DE_EPAGES::AppStore::API::Object::AppsClient->create( {
            BaseURL  => 'http://localhost:8088/rs-test/',
            Shop     => $Shop,
            ShopType => 'eCom100',
            Locale   => 'en_GB',
            BusinessUnit => 'blabla',
        }
    );
    return;
}

sub test {
    my $self = shift;
    $self->testGetAppList();
    $self->testGetSingleApp();
    $self->testPostAppConsent();
    $self->testMyApps();
    $self->testFindAppIdByUrl();
    $self->testGetAppActionsUninstallApp();
}

sub testFindAppIdByUrl() {
    my $self = shift;

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertFindAppIdByUrlContent, } );

    my $responseId = $Client->findAppIdByUrl('SOAP_APP','onesaas.com');
    is( $responseId, '54e207786a1a63f3019bb67b', 'correct app id found from appUrl' );
}

sub testGetAppActionsUninstallApp() {
    my $self = shift;

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppActions, } );

    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_appLinksUrl("333"), qr#appstore/shopGUID/installations/333/actions#, 'get app action url ok' );

    my $response = $Client->getAppLinks(333);

    my  @aLinksActions = $response->{'links'};
    is( $aLinksActions[0][0]->{'rel'}, 'installApp', 'correct rel of installApp link' );
    is( $aLinksActions[0][1]->{'rel'}, 'uninstallApp', 'correct rel of uninstallApp link' );
    is( $aLinksActions[0][2]->{'rel'}, 'openApp', 'correct rel of openApp link' );

    is( $aLinksActions[0][0]->{'href'}, 'http://localhost:8088/rs/appstore/shopGUID/installations/333', 'correct installApp link location' );
    is( $aLinksActions[0][1]->{'href'}, 'http://localhost:8088/rs/appstore/shopGUID/installations/333', 'correct uninstallApp link location' );
    is( $aLinksActions[0][2]->{'href'}, 'http://localhost:8088/rs/appstore/shopGUID/installations/333', 'correct openApp link location' );

}

sub testPostAppConsent {
    my $self = shift;

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'post' => \&_assertPostAppConsentContent, } );

    # check url
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_installAppUrl("333"), qr#appstore/shopGUID/installations/333#, 'post app consent url OK' );

    # post app consent
    my $response = $Client->submitAppConsent(333,"products_read");

    is( $response->{'shopId'},    'sID',     'correct shopId');
    is( $response->{'appId'}, 'aID', 'correct appId' );
    is( $response->{'callbackUrl'},   'http://callback', 'correct callbackUrl' );

}

sub testGetSingleApp {
    my $self = shift;

    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppContent, } );

    # check url
    ## no critic (ProhibitPrivateSubroutineUse)
    like( $Client->_singleAppUrl("333aaa"), qr#appstore/shopGUID/apps/333aaa#, 'single appstore app url OK' );

    # get single app
    my $privateApp = $Client->getApp(333);
    is( $privateApp->{'name'},      'bourneIdentityGET',         'App name is set to: ' . $privateApp->{'name'} );
    is( $privateApp->{'logoUrl'},   'http://hostaname/logo.jpg', 'App logo is set to: ' . $privateApp->{'logoUrl'} );
    is( $privateApp->{'price'},     'free',                      'App price is set to: ' . $privateApp->{'price'} );
    is( $privateApp->{'developer'}, 'mister developer',          'App price is set to: ' . $privateApp->{'developer'} );
    my @aRoles = $privateApp->{'scopes'};
    is( $aRoles[0][0], 'products_read',          'App first role is set to: ' . $aRoles[0][0] );
}

sub testGetAppList {
    my $self   = shift;
    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetAppListContent, } );

    like( $Client->_appsUrl(), qr#appstore/shopGUID/apps#, 'multiple appstore apps url OK' ); ## no critic (ProhibitPrivateSubroutineUse)

    # get list apps
    my @aList = $Client->listApps();
    is( $aList[0][0]->{'name'}, 'KittenApp', 'App list first name is :' . $aList[0][0]->{'name'} );
}

sub testMyApps {
    my $self   = shift;
    my $Client = $self->{'Client'};
    $Client->{'HttpClient'} = _mockHTTPClient( { 'get' => \&_assertGetMyAppsContent, } );

    like( $Client->_myAppsUrl(), qr#appstore/shopGUID/installations#, 'my apps url OK' ); ## no critic (ProhibitPrivateSubroutineUse)

    # get my apps
    my @aList = $Client->myApps();
    is( $aList[0][0]->{'name'}, 'KittenApp', 'My First App is :' . $aList[0][0]->{'name'} );
    is( $aList[0][1]->{'name'}, 'KittenApp2', 'My Second App is :' . $aList[0][1]->{'name'} );
    is( $aList[0][0]->{'logoUrl'}, 'http://somehost/someLogo.png', 'My First App Logo is :' . $aList[0][0]->{'logoUrl'} );
    is( $aList[0][1]->{'logoUrl'}, 'http://somehost/someLogo2.png', 'App Second App Logo is :' . $aList[0][1]->{'logoUrl'} );
}

sub _mockHTTPClient {
    my ($hMockMethods) = @_;
    my $mockRemote = Test::MockObject->new();
    $mockRemote->set_isa('DE_EPAGES::EPagesJ::API::Object::HTTPClient');
    $mockRemote->fake_module( 'DE_EPAGES::EPagesJ::API::Object::HTTPClient', %$hMockMethods );
    while ( my ( $method, $handler ) = each %$hMockMethods ) {
        $mockRemote->mock( $method, $handler );
    }
    return $mockRemote;
}

sub _assertGetAppActions {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    ## no critic (ProhibitComplexRegexes)
    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/installations/333/actions|, 'URL' );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
    {
        "links": [
        {
            "rel": "installApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        },{
            "rel": "uninstallApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        },{
            "rel": "openApp",
            "href": "http://localhost:8088/rs/appstore/shopGUID/installations/333",
            "title": null
        }
  ]
}
EOL
    );
}

sub _assertPostAppConsentContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    ## no critic (ProhibitComplexRegexes)
    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/installations/333|, 'URL' );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
       {
                "shopId":"sID",
                "appId":"aID",
                "callbackUrl":"http://callback",
                "state":"PENDING"
       }
EOL
    );

}

sub _assertGetAppContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/apps/333|, 'URL' );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
       {
            "id": "333",
            "scopes": [ "products_read" ],
            "iconUrl": "http://i.imgur.com/dKXXmmKs.png",
            "logoUrl": "http://hostaname/logo.jpg",
            "name": "bourneIdentityGET",
            "shortDescription": "short description",
            "video": "http://video.url",
            "images": [
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/dKXXmmKt.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/dKXXmmK.png",
                            "classifier": "large"
                        }
                    ]
                },
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/jkvFOEet.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/jkvFOEe.png",
                            "classifier": "large"
                        }
                    ]
                },
                {
                    "image": [
                        {
                            "url": "http://i.imgur.com/CCRbV9Ft.png",
                            "classifier": "small"
                        },
                        {
                            "url": "http://i.imgur.com/CCRbV9F.png",
                            "classifier": "large"
                        }
                    ]
                }
            ],
            "features": [
                "feature1",
                "feature2"
            ],
            "description": "description",
            "price": "free",
            "developer": "mister developer",
            "requirements": "requirement 1, requirement 2",
            "support": {
                "email": "support@comp.any",
                "website": "http://comp.any"
            },
            "links": []
        }
EOL
    );
}

sub _assertGetAppListContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    like( $URL, qr|^http://localhost:8088/rs-test/appstore/shopGUID/apps|, 'URL' );
    like( $URL, qr|shopType=eCom100|,                             'shopType' );
    like( $URL, qr|locale=en_GB|,                                 'locale' );
    like( $URL, qr|page=0|,                                       'page' );
    like( $URL, qr|businessUnit=blabla|,                          'businessUnit' );

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
[
    {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "shopGUID": "developerguid",
           "callbackUrl": "callback",
           "notificationUrl": "notification",
           "logoUrl": "http://somehost/someLogo.png",
           "price": "free",
           "shopType": "eCom100"
   }
]
EOL
    );
}

sub _assertGetMyAppsContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
[
    {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "logoUrl": "http://somehost/someLogo.png"
    },
    {
           "id": "54e207786a1a63f3019bb67c",
           "name": "KittenApp2",
           "logoUrl": "http://somehost/someLogo2.png"
    },
    {
           "id": "54e207786a1a63f3019bb67d",
           "name": "KittenApp3",
           "logoUrl": "http://somehost/someLogo3.png"
    }
]
EOL
    );
}

sub _assertFindAppIdByUrlContent {
    shift;
    my ( $URL, $hHeader, $Content ) = @_;

    return HTTP::Response->new( HTTP_OK, status_message(HTTP_OK), undef, <<'EOL'
        {
           "id": "54e207786a1a63f3019bb67b",
           "name": "KittenApp",
           "logoUrl": "http://somehost/someLogo.png"
        }
EOL
    );
}

__PACKAGE__->runScript;
done_testing();
