#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::CancelTaxSender
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  test DE_EPAGES::AvaTax::API::CancelTaxSender
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::CancelTaxSender;
use base DE_EPAGES::Core::API::Test;

use Test::Exception;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::MockObject;
use DE_EPAGES::Core::API::TestParameter qw (TestObject);
use DE_EPAGES::Test::API::TestTools qw (TestExistsMethod);
use DE_EPAGES::Core::API::PerlTools qw ( UsePackage );
use DE_EPAGES::AvaTax::API::CancelTaxSender;
use DE_EPAGES::AvaTax::API::Constants qw (AVATAX_DOCUMENTTYPE_SalesInvoice AVATAX_CANCELTAX_SUCCESS );

use strict;

sub test {
    my $self = shift;

    $self->test_sendCancalTax_SuccessfulResponse();
    $self->test_sendCancalTax_NotSuccessfulResponse();
    $self->test_sendCancalTax_WrongResponseObject();

    done_testing();
}



sub test_sendCancalTax_SuccessfulResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ISAReturn = 1;
    my $RemoteTaxSvc = $self->_buildSuccessfulResponse();
    my $CancelTaxSender = $self->_createCancelTaxSender($RemoteTaxSvc, $ISAReturn);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_createOrderExtractor($LineItemContainer);
    my $Result = $CancelTaxSender->sendCancelTax($OrderExtractor);

    is($Result,undef,"$SubTestName - tests if sendCancelTax is successful");

    my $CancelTaxRequest = $self->{'param_cancelTax'}; # get the parameter from cancelTax call
    is($CancelTaxRequest->CompanyCode(),'Test-Company',"$SubTestName - test CompanyCode return value");
    is($CancelTaxRequest->DocType(),AVATAX_DOCUMENTTYPE_SalesInvoice,"$SubTestName - test DocType return value");
    is($CancelTaxRequest->DocCode(),'Test-Doc-Code-123',"$SubTestName - test DocCode return value");
    is($CancelTaxRequest->CancelCode(),'DocVoided',"$SubTestName - test CancelCode return value");

    is($LineItemContainer->get('AvaTax_StatusResponse'), AVATAX_CANCELTAX_SUCCESS, "$SubTestName - tests if Status is set correctly");
    is($LineItemContainer->get('AvaTax_StatusMessage'), 'message', "$SubTestName - tests if StatusMesssage is set correctly");
    is($LineItemContainer->get('AvaTax_TimeStamp'), 'now', "$SubTestName - tests if TimeStamp is set correctly");

    $self->{'param_getTax'} = undef; #rollback
}



sub test_sendCancalTax_NotSuccessfulResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ISAReturn = 1;
    my $CancelTaxRequest = $self->_buildFailedResponse();
    my $CancelTaxSender = $self->_createCancelTaxSender($CancelTaxRequest, $ISAReturn);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_createOrderExtractor($LineItemContainer);
    throws_ok( sub { $CancelTaxSender->sendCancelTax($OrderExtractor) },
                   qr/CancelTaxResultFailed/,
                   "$SubTestName - tests if sendCancelTax throw error in case of failed"
         );
}



sub test_sendCancalTax_WrongResponseObject {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ISAReturn = 0;
    my $CancelTaxRequest = $self->_buildSuccessfulResponse();
    my $CancelTaxSender = $self->_createCancelTaxSender($CancelTaxRequest, $ISAReturn);

    my $LineItemContainer = $self->_createLineItemContainer();
    my $OrderExtractor = $self->_createOrderExtractor($LineItemContainer);
    throws_ok( sub { $CancelTaxSender->sendCancelTax($OrderExtractor) },
                   qr/CancelTaxResultFailed/,
                   "$SubTestName - tests if sendCancelTax throw error in case of failed"
         );
}



#========================================================================================
# Helpers
#========================================================================================
sub _createCancelTaxSender {
    my $self = shift;
    my ($RemoteTaxSvc, $ISAReturn) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::CancelTaxSender',
        []
    );

    my $CancelTaxRequest = DE_EPAGES::AvaTax::API::DTO::CancelTaxRequest->new();
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_connectToTaxService', $RemoteTaxSvc , [{'object' => 'DE_EPAGES::Shop::API::Object::Shop'}]);
    $MockifyObject->addMockWithReturnValue('_createCancelTaxRequest', $CancelTaxRequest);
    $MockifyObject->addMockWithReturnValue('_getCurrentDateTime', 'now');
    $MockifyObject->addMockWithReturnValueAndParameterCheck(
        '_IsA',
        $ISAReturn,
        [
            {'object' => 'DE_EPAGES::AvaTax::API::DTO::CancelTaxResult'},
            {'string' => 'DE_EPAGES::AvaTax::API::DTO::CancelTaxResult'}
        ]
    );
    $MockifyObject->addMockWithReturnValueAndParameterCheck('_createResponseParser', $self->_createResponseParser(), [
        {'object' => 'DE_EPAGES::AvaTax::API::DTO::CancelTaxResult'}
    ]);

    return $MockifyObject->getMockObject();
}



sub _createOrderExtractor {
    my $self = shift;
    my ($LineItemContainer) = @_;

    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::OrderExtractor',
        []
    );
    $MockifyObject->addMockWithReturnValue('getShop', $self->_createShop());
    $MockifyObject->addMockWithReturnValue('getLineItemContainer', $LineItemContainer);
    $MockifyObject->addMockWithReturnValue('getAvaTaxCompanyCode', 'Test-Company');

    return $MockifyObject->getMockObject();
}



sub _createShop {
    my $self = shift;

    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', {});
    return $Shop;
}

sub _createLineItemContainer {
    my $self = shift;
    my $hAttributes = {
        'AvaTax_DocCode' => 'Test-Doc-Code-123',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
}



sub _buildSuccessfulResponse {
    my $self = shift;
    my $CancelTaxResult = DE_EPAGES::AvaTax::API::DTO::CancelTaxResult->new();
    $CancelTaxResult->ResultCode('Success');
    my $Response = $self->_createRemoteConnector($CancelTaxResult);
    return $Response;
}



sub _buildFailedResponse {
    my $self = shift;

    my $CancelTaxResult = DE_EPAGES::AvaTax::API::DTO::CancelTaxResult->new();
    $CancelTaxResult->ResultCode('Failed');
    my $Response = $self->_createRemoteConnector($CancelTaxResult);
    return $Response;
}



sub _createRemoteConnector {
    my $self = shift;
    my ($CancelTaxResponse) = @_;

    UsePackage( 'DE_EPAGES::AvaTax::API::RemoteTaxSvc' );
    TestExistsMethod('DE_EPAGES::AvaTax::API::RemoteTaxSvc', 'cancelTax');
    my $Mock = Test::MockObject->new();
    $Mock->mock('cancelTax', sub {
        my $mockedself = shift;
        my ($Request) = @_;
        $self->{'param_cancelTax'} = $Request;
        TestObject('cancelTaxRequest object', $Request, 'DE_EPAGES::AvaTax::API::DTO::CancelTaxRequest');
        return $CancelTaxResponse;
    });

    return $Mock;
}

sub _createResponseParser {
    my $self = shift;
    my $MockifyObject = DE_EPAGES::Test::Mock::Mockify->new(
        'DE_EPAGES::AvaTax::API::ResponseParser',
        []
    );
    $MockifyObject->addMockWithReturnValue('getStatusCancelTax', AVATAX_CANCELTAX_SUCCESS);
    $MockifyObject->addMockWithReturnValue('getStatusMessage', 'message');
    return $MockifyObject->getMockObject();
}

__PACKAGE__->runScript();