#========================================================================================
# package      DE_EPAGES::AvaTax::t::unit::OrderExtractor
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  unit test for DE_EPAGES::AvaTax::API::OrderExtractor
#========================================================================================
package DE_EPAGES::AvaTax::t::unit::OrderExtractor;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::AvaTax::API::OrderExtractor;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Test::Mock::Mockify;
use Test::Exception;
use Test::More;

use strict;



sub test {
    my $self = shift;

    $self->successfulExtractionWithBasket();
    $self->successfulExtractionWithOrderAndCustomerVATID();
    $self->successfulExtractionWithOrderAndWithoutCustomerVATID();
    $self->successfulExtractionWithRegisteredCustomer();

    $self->failExtractionNotBasketOrOrder();

    $self->failExtractionWithoutLineItemContainer();
    $self->failExtractionMissingCurrencyID();
    $self->failExtractionWrongTaxModel();

    $self->failExtractionMissingShop();

    $self->failExtractionMissingTaxMethodAvaTax();
    $self->failExtractionNotInstanceOfTaxMethodAvaTax();
    $self->failExtractionMissingAvaTaxCredentials();

    $self->failExtractionNotInstanceOfShopAddress();
    $self->failExtractionMissingShopAddressInfo();

    $self->failExtractionNotInstanceOfCustomerAddress();
    $self->failExtractionMissingCustomerAddressInfo();

    $self->test_hashCalculationCorrectHash();
    $self->test_hashCalculationUndefHash();
    $self->test_hashCalculationDifferentHash();

    $self->test_hashCalculationUpdateHash();

    done_testing();
}
## no critic (ProhibitLongLines)
#----------------------------------------------------------------------------------------
sub successfulExtractionWithBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $LineItemContainer = $self->_createLineItemContainer();
    my $Basket = $self->_createBasket($LineItemContainer);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was successful extracted from the basket");

    is($BasketExtractor->getErrorMessage(), undef,"$SubTestName - Test if any errorMessage was set");
    is_deeply($BasketExtractor->getShopAddress(), $self->_expectedShopAddressObject(),"$SubTestName - Test if the ShopAddress was correctly set");
    is_deeply($BasketExtractor->getCustomerAddress(), $self->_expectedCustomerAddressObject(),"$SubTestName - Test if the CustomerAddress was correctly set");
    is($BasketExtractor->getCustomerCode(), 'TaxSvcTest',"$SubTestName - Test if the CustomerCode was correctly set");
    is($BasketExtractor->getAvaTaxCompanyCode(), 'TestCompanyCode',"$SubTestName - Test if the CompanyCode was correctly set");
    is_deeply($BasketExtractor->getContainerAttributes(), $self->_expectedLineItemContainerAttributes(),"$SubTestName - Test if the LineItemContainer attributes were correctly set");
    is($BasketExtractor->getDiscountValue(), '10',"$SubTestName - Test if the discount value was correctly set");
    is_deeply($BasketExtractor->getLines(), [],"$SubTestName - Test if the Avatax Lines are set correctly");
    ok(!$BasketExtractor->isPaid(), "$SubTestName - tests if the paid flag is not set.");
    is_deeply($BasketExtractor->getShop()->alias(), 'TestShop', "$SubTestName - tests if the correct shop was returned.");
    is($BasketExtractor->getOrderAlias(), 'MyBasket',"$SubTestName - Test if basket alias was returned.");
}

#----------------------------------------------------------------------------------------
sub successfulExtractionWithOrderAndCustomerVATID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $LineItemContainer = $self->_createLineItemContainer();
    my $Order = $self->_createOrder($LineItemContainer, $self->_buildCustomerWithVATID());

    my $OrderExtractor = $self->_createOrderExtractor();
    ok($OrderExtractor->extractDataFrom($Order),"$SubTestName - Test if the Data was successful extracted from the order");

    is($OrderExtractor->getErrorMessage(), undef,"$SubTestName - Test if any errorMessage was set");
    is_deeply($OrderExtractor->getShopAddress(), $self->_expectedShopAddressObject(),"$SubTestName - Test if the ShopAddress was correctly set");
    is_deeply($OrderExtractor->getCustomerAddress(), $self->_expectedCustomerAddressObject(),"$SubTestName - Test if the CustomerAddress was correctly set");
    is($OrderExtractor->getCustomerCode(), 'CustomerNumber',"$SubTestName - Test if the CustomerCode was correctly set");
    is($OrderExtractor->getAvaTaxCompanyCode(), 'TestCompanyCode',"$SubTestName - Test if the CompanyCode was correctly set");
    is_deeply($OrderExtractor->getContainerAttributes(), $self->_expectedLineItemContainerAttributes(),"$SubTestName - Test if the LineItemContainer attributes were correctly set");
    is($OrderExtractor->getDiscountValue(), '10',"$SubTestName - Test if the discount value was correctly set");
    is_deeply($OrderExtractor->getLines(), [],"$SubTestName - Test if the Avatax Lines are set correctly");
    ok(!$OrderExtractor->isPaid(), "$SubTestName - tests if the paid flag is not set.");
    is_deeply($OrderExtractor->getShop()->alias(), 'TestShop', "$SubTestName - tests if the correct shop was returned.");
    is($OrderExtractor->getOrderAlias(), 'MyOrder',"$SubTestName - Test if order alias was returned.");
    is($OrderExtractor->getCustomerVATID(), $self->_getTestVATID(),"$SubTestName - Test if customer VAT ID was returned.");
}

#----------------------------------------------------------------------------------------
sub successfulExtractionWithOrderAndWithoutCustomerVATID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $LineItemContainer = $self->_createLineItemContainer();
    my $Order = $self->_createOrder($LineItemContainer, $self->_buildCustomerWithoutVATID());

    my $OrderExtractor = $self->_createOrderExtractor();
    ok($OrderExtractor->extractDataFrom($Order),"$SubTestName - Test if the Data was successful extracted from the order");

    is($OrderExtractor->getErrorMessage(), undef,"$SubTestName - Test if any errorMessage was set");
    is_deeply($OrderExtractor->getShopAddress(), $self->_expectedShopAddressObject(),"$SubTestName - Test if the ShopAddress was correctly set");
    is_deeply($OrderExtractor->getCustomerAddress(), $self->_expectedCustomerAddressObject(),"$SubTestName - Test if the CustomerAddress was correctly set");
    is($OrderExtractor->getCustomerCode(), 'CustomerNumber',"$SubTestName - Test if the CustomerCode was correctly set");
    is($OrderExtractor->getAvaTaxCompanyCode(), 'TestCompanyCode',"$SubTestName - Test if the CompanyCode was correctly set");
    is_deeply($OrderExtractor->getContainerAttributes(), $self->_expectedLineItemContainerAttributes(),"$SubTestName - Test if the LineItemContainer attributes were correctly set");
    is($OrderExtractor->getDiscountValue(), '10',"$SubTestName - Test if the discount value was correctly set");
    is_deeply($OrderExtractor->getLines(), [],"$SubTestName - Test if the Avatax Lines are set correctly");
    ok(!$OrderExtractor->isPaid(), "$SubTestName - tests if the paid flag is not set.");
    is_deeply($OrderExtractor->getShop()->alias(), 'TestShop', "$SubTestName - tests if the correct shop was returned.");
    is($OrderExtractor->getOrderAlias(), 'MyOrder',"$SubTestName - Test if order alias was returned.");
    is($OrderExtractor->getCustomerVATID(), undef,"$SubTestName - Test if customer VAT ID is empty.");
}

#----------------------------------------------------------------------------------------
sub successfulExtractionWithRegisteredCustomer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Basket = $self->_createBasketWithRegisterCustomer();

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was successful extracted from the basket");

    is($BasketExtractor->getErrorMessage(), undef,"$SubTestName - Test if any errorMessage was set");

    ok(!$BasketExtractor->isPaid(), "$SubTestName - tests if the paid flag is not set.");
    is_deeply($BasketExtractor->getShop()->alias(), 'TestShop', "$SubTestName - tests if the correct shop was returned.");
    is($BasketExtractor->getOrderAlias(), 'MyBasket',"$SubTestName - Test if basket alias was returned.");
}

#----------------------------------------------------------------------------------------
sub failExtractionNotBasketOrOrder {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $NotBasketOrOrder = DE_EPAGES::Test::Mock::EpagesObject->new('Test::Object', {'instanceOf' => 'NotBasketOrOrderObject'});

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($NotBasketOrOrder),"$SubTestName - Test if the Data was NOT extracted from the basket");
    is($BasketExtractor->getBasketOrOrder(), undef, "$SubTestName - Test if the BasketOrOrder was not defined");
}

#----------------------------------------------------------------------------------------
sub failExtractionWithoutLineItemContainer {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $LineItemContainer = undef;
    my $Basket = $self->_createBasket($LineItemContainer);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $LineItemContainerFromExtractor = $BasketExtractor->getLineItemContainer();
    is($LineItemContainerFromExtractor, undef, "$SubTestName - Test if the LineItemContainer was not defined");
}


#----------------------------------------------------------------------------------------
sub failExtractionMissingCurrencyID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $NetTaxModel = 0;

    my $hAttributes = {
        'CurrencyID' => undef,
        'TaxModel' => $NetTaxModel,
        'LanguageID' => 'ID',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
    my $Basket = $self->_createBasket($LineItemContainer);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    is( $BasketExtractor->getContainerAttributes(), undef, "$SubTestName - Test if the LineItemContainer does not have CurrencyID");
    my $hExpectedErrorMessage = { 'Code' => 'CurrencyIDMismatch', 'Info' => { 'CurrencyID' => undef } };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if the LineItemContainer does not have CurrencyID");

}


#--------------------------------------------------------------------------------------
sub failExtractionWrongTaxModel {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $BruttoTaxModel = 1;

    my $hAttributes = {
        'CurrencyID' => 'USD',
        'TaxModel' => $BruttoTaxModel,
        'LanguageID' => 'ID',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
    my $Basket = $self->_createBasket($LineItemContainer);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    is( $BasketExtractor->getContainerAttributes(), undef, "$SubTestName - Test if the LineItemContainer does not have CurrencyID");
    my $hExpectedErrorMessage = { 'Code' => 'TaxModelMismatch', 'Info' => {} };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if the LineItemContainer does not have CurrencyID");
}
#--------------------------------------------------------------------------------------
sub failExtractionMissingShop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $Shop = undef;
    my $Basket = $self->_createBasketWithShop($Shop);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = {
        'Code' => 'PARAMETER_MISSING',
        'Info' => { 'Object' => undef }
    };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if the Shop is undef");
}

#--------------------------------------------------------------------------------------
sub failExtractionMissingTaxMethodAvaTax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $TaxMethodAvaTax = undef;
    my $Shop = $self->_createShop($TaxMethodAvaTax);
    my $Basket = $self->_createBasketWithShop($Shop);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = { 'Code' => 'NoTaxMethod', 'Info' => { 'ShopAlias' => 'TestShop'} };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if the TaxMethod is undef");
}
#--------------------------------------------------------------------------------------
sub failExtractionNotInstanceOfTaxMethodAvaTax {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = { 'instanceOf' => 'NotTaxMethodAvaTax' };
    my $NotInstanceOfTaxMethodAvaTax = DE_EPAGES::Test::Mock::EpagesObject->new('Test::Object', $hAttributes);
    my $Shop = $self->_createShop($NotInstanceOfTaxMethodAvaTax);
    my $Basket = $self->_createBasketWithShop($Shop);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = { 'Code' => 'IsNotAvaTax', 'Info' => { 'ShopAlias' => 'TestShop'} };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if the TaxMethod is not TaxMethodAvaTax");
}
#--------------------------------------------------------------------------------------
sub failExtractionMissingAvaTaxCredentials {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {
        'instanceOf' => 'TaxMethodAvaTax',
        'AvaTax_AccountNumber' => undef,
        'AvaTax_LicenseKey' => 'TestLicenseKey',
        'AvaTax_CompanyCode' => 'TestCompanyCode',
    };
    my $TaxMethodAvaTax = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax', $hAttributes);
    my $Shop = $self->_createShop($TaxMethodAvaTax);
    my $Basket = $self->_createBasketWithShop($Shop);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = { 'Code' => 'MissingCredentials',
        'Info' => { 'AvaTax_CompanyCode' => 'TestCompanyCode', 'AvaTax_LicenseKey' => 'TestLicenseKey', 'AvaTax_AccountNumber' => undef }
    };
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if are missing credentials for avatax");
    is($BasketExtractor->getAvaTaxCompanyCode(), undef,"$SubTestName - Test if the gethTaxMethodAvaTax is undef");
}

#--------------------------------------------------------------------------------------
sub failExtractionNotInstanceOfShopAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShopAddress = DE_EPAGES::Test::Mock::EpagesObject->new('Test::Object', {});
    my $Basket = $self->_createBasketWithShopAddress($ShopAddress);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $ErrorMessage = $BasketExtractor->getErrorMessage()->{'Code'};
    is($ErrorMessage, 'WRONG_CLASS',"$SubTestName - Test if ShopAddress is not a object of DE_EPAGES::Address::API::Object::Address");
    is($BasketExtractor->getShopAddress(), undef,"$SubTestName - Test if the ShopAddress is undef");
}
#--------------------------------------------------------------------------------------
sub failExtractionMissingShopAddressInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $hAttributes = {
        'Company' => 'ShopTest Company',
        'FullName' => 'ShopTest FullName',
        'Street' => 'ShopTest street',
        'Street2' => 'ShopTest street2',
        'Zipcode' => undef,
        'City' => 'ShopTestTown',
        'State' => 'ShopTest State',
        'Country' => {'Code2' => 'US'},
    };

    my $ShopAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);
    my $Basket = $self->_createBasketWithShopAddress($ShopAddress);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = $self->_expectedErrorMissingShopAddress();
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if ShopAddress miss some address information");
    is($BasketExtractor->getShopAddress(), undef,"$SubTestName - Test if the ShopAddress is undef");
}
#--------------------------------------------------------------------------------------
sub failExtractionNotInstanceOfCustomerAddress {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('Test::Object', {});
    my $BillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('Test::Object', {});
    my $Basket = $self->_createBasketWithShippingAddress($ShippingAddress, $BillingAddress);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $ErrorMessage = $BasketExtractor->getErrorMessage()->{'Code'};
    is($ErrorMessage, 'WRONG_CLASS',"$SubTestName - Test if ShopAddress is not a object of DE_EPAGES::Address::API::Object::Address");
    is($BasketExtractor->getCustomerAddress(), undef,"$SubTestName - Test if the ShopAddress is undef");
}
#--------------------------------------------------------------------------------------
sub failExtractionMissingCustomerAddressInfo {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $ShippingAddress = undef;
    my $hAttributes = {
            'Company' => 'CustomerTest Company',
            'FullName' => 'CustomerTest FullName',
            'Street' => 'CustomerTest street',
            'Street2' => 'CustomerTest street2',
            'Zipcode' => 'Test Zipcode',
            'City' => 'CustomerTestTown',
            'State' => 'CustomerTest State',
            'Country' => {'Code2' => 'Not US'},
    };

    my $BillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);
    my $Basket = $self->_createBasketWithShippingAddress($ShippingAddress, $BillingAddress);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok(!$BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was NOT extracted from the basket");

    my $hExpectedErrorMessage = $self->_expectedErrorMissingCustomerAddress();
    is_deeply($BasketExtractor->getErrorMessage(), $hExpectedErrorMessage,"$SubTestName - Test if CustomerAddress miss some address information");
    is($BasketExtractor->getCustomerAddress(), undef,"$SubTestName - Test if the ShopAddress is undef");
}
#--------------------------------------------------------------------------------------
sub test_hashCalculationCorrectHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];
    my $LineItemContainer = $self->_createLineItemContainer();
    my $Basket = $self->_createBasket($LineItemContainer);

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket),"$SubTestName - Test if the Data was successful extracted from the basket");

    is($BasketExtractor->hasChanged(), 0, "$SubTestName - Test if hash has not changed");
}
#--------------------------------------------------------------------------------------
sub test_hashCalculationUndefHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasketForAvaTax();

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket), "$SubTestName - Test if the Data was successful extracted from the basket");

    is($BasketExtractor->hasChanged(), 1, "$SubTestName - Test if hash has changed");
}
#--------------------------------------------------------------------------------------
sub test_hashCalculationDifferentHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasketForAvaTax();

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket), "$SubTestName - Test if the Data was successful extracted from the basket");

    is($BasketExtractor->hasChanged(), 1, "$SubTestName - Test if hash has changed");
}
#--------------------------------------------------------------------------------------
sub test_hashCalculationUpdateHash {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Basket = $self->_createBasketForAvaTax();

    my $BasketExtractor = $self->_createOrderExtractor();
    ok($BasketExtractor->extractDataFrom($Basket), "$SubTestName - Test if the Data was successful extracted from the basket");

    $BasketExtractor->updateHash();

    is($BasketExtractor->hasChanged(), 0, "$SubTestName - Test if no change was detected since the current hash was recently stored.");
}
#--------------------------------------------------------------------------------------



#--------------------------------------------------------------------------------------
# Help methods
#--------------------------------------------------------------------------------------
sub _createOrderExtractor {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::OrderExtractor', [] );
    $Mockify->addMockWithReturnValue('_createDiscountObject', $self->_createDiscountObject());
    $Mockify->addMockWithReturnValue('_createLineItemObject', $self->_createLineItemObject());
    $Mockify->addMockWithReturnValueAndParameterCheck('_CalculateAvataxChecksum', 'hashCode', [{'object' => 'DE_EPAGES::AvaTax::API::OrderExtractor'}]);

    return $Mockify->getMockObject();
}

sub _createDiscountObject {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::Discount', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('getDiscountValue',10,[{'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer'}]); ## no critic (ProhibitMagicNumbers)

    return $Mockify->getMockObject();
}

sub _createLineItemObject {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::AvaTax::API::LineItem', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('getLines',[],[{'object' => 'DE_EPAGES::Order::API::Object::LineItemContainer'},'string']);

    return $Mockify->getMockObject();
}

sub _createBasketWithShopAddress {
    my $self = shift;
    my ($ShopAddress) = @_;
    my $hAttributes = {
        'instanceOf' => 'Basket',
        'Customer' => undef,
        'Site' => $self->_createShop($self->_createTaxMethodAvaTax()),
        'LineItemContainer' => $self->_createLineItemContainer(),
        'ShippingAddress' => $self->_createShippingAddress(),
        'ShopAddress' => $ShopAddress,
    };
    my $MockBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    return $MockBasket;
}


sub _createBasketWithShippingAddress {
    my $self = shift;
    my ($ShippingAddress, $BillingAddress) = @_;
    my $hAttributes = {
        'instanceOf' => 'Basket',
        'Customer' => undef,
        'Site' => $self->_createShop($self->_createTaxMethodAvaTax()),
        'LineItemContainer' => $self->_createLineItemContainer(),
        'ShippingAddress' => $ShippingAddress,
        'BillingAddress' => $BillingAddress,
        'ShopAddress' => $self->_createShopAddress(),
    };
    my $MockBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    return $MockBasket;
}


sub _createBasketWithShop {
    my $self = shift;
    my ($Shop) = @_;
    my $hAttributes = {
        'instanceOf' => 'Basket',
        'Customer' => undef,
        'Site' => $Shop,
        'LineItemContainer' => $self->_createLineItemContainer(),
        'ShippingAddress' => $self->_createShippingAddress(),
        'ShopAddress' => $self->_createShopAddress(),
    };
    my $MockBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    return $MockBasket;
}

sub _createBasket {
    my $self = shift;
    my ($LineItemContainer) = @_;
    my $hAttributes = {
        'Alias' => 'MyBasket',
        'instanceOf' => 'Basket',
        'Customer' => undef,
        'Site' => $self->_createShop($self->_createTaxMethodAvaTax()),
        'LineItemContainer' => $LineItemContainer,
        'ShippingAddress' => $self->_createShippingAddress(),
        'ShopAddress' => $self->_createShopAddress(),
        'PaidOn' => undef,
    };
    my $MockBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    $MockBasket->mock('class', sub {
        return  $self->_createClass();
    });
    return $MockBasket;
}

sub _createClass {
    my $self = shift;
    my $hAttributes = {};
    my $MockedClass = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Class', $hAttributes);
    $MockedClass->mock('existsAttribute', sub {
        return 0;
    });
    return $MockedClass;
}

sub _createBasketWithRegisterCustomer {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyBasket',
        'instanceOf' => 'Basket',
        'Customer' => $self->_buildCustomerWithVATID(),
        'Site' => $self->_createShop($self->_createTaxMethodAvaTax()),
        'LineItemContainer' => $self->_createLineItemContainer(),
        'ShippingAddress' => $self->_createShippingAddress(),
        'ShopAddress' => $self->_createShopAddress(),
        'PaidOn' => undef,
    };
    my $MockBasket = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::Basket', $hAttributes);
    $MockBasket->mock('class', sub {
        return  $self->_createClass();
    });
    return $MockBasket;
}

sub _buildCustomerWithVATID {
    my $self = shift;
    my $hSettings = {
        'Alias' => 'CustomerNumber',
        'BillingAddress' => $self->_createBillingAddress($self->_getTestVATID()),
        'AvataxTaxExemption' => undef,
    };
    return $self->_createCustomer($hSettings);
}

sub _buildCustomerWithoutVATID {
    my $self = shift;
    my $hSettings = {
        'Alias' => 'CustomerNumber',
        'BillingAddress' => $self->_createBillingAddress(),
        'AvataxTaxExemption' => undef,
    };
    return $self->_createCustomer($hSettings);
}

sub _createCustomer {
    my $self = shift;
    my ($hAttributes) = @_;
    my $MockedCustomer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Customer::API::Object::Customer', $hAttributes);
    $MockedCustomer->mock('class', sub {shift; return $self->_createCustomerClass() });
    return $MockedCustomer;
}

sub _createCustomerClass{
    my $self = shift;
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new( 'DE_EPAGES::Object::API::Object::Class', [] );
    $Mockify->addMockWithReturnValueAndParameterCheck('existsAttribute', 1 ,[{'string' => 'AvataxTaxExemption'}]);
    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($TaxMethodAvaTax) = @_;
    my $hAttributes = {
        'Alias' => 'TestShop',
        'TaxMethod' => $TaxMethodAvaTax,
    };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Shop::API::Object::Shop', $hAttributes);
    return $Shop;
}

sub _createTaxMethodAvaTax {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'TaxMethodAvaTax',
        'AvaTax_AccountNumber' => 'TestAccountNumber',
        'AvaTax_LicenseKey' => 'TestLicenseKey',
        'AvaTax_CompanyCode' => 'TestCompanyCode',
    };
    my $TaxMethodAvaTax = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AvaTax::API::Object::TaxMethodAvaTax', $hAttributes);
    return $TaxMethodAvaTax;
}

sub _createLineItemContainer {
    my $self = shift;
    my $NetTaxModel = 0;

    my $hAttributes = {
        'instanceOf' => 'LineItemContainer',
        'AvaTax_Checksum' => 'hashCode',
        'CurrencyID' => 'USD',
        'TaxModel' => $NetTaxModel,
        'LanguageID' => 'ID',
        'Children' => [$self->_createLineItemAvaTax()],
    };
    my $MockedLineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
    return $MockedLineItemContainer;
}

sub _createShippingAddress {
    my $self = shift;

    my $hAttributes = {
        'Company' => 'CustomerTest Company',
        'FullName' => 'CustomerTest FullName',
        'Street' => 'CustomerTest street',
        'Street2' => 'CustomerTest street2',
        'Zipcode' => '50000',
        'City' => 'CustomerTestTown',
        'State' => 'CustomerTest State',
        'Country' => {'Code2' => 'US'},
    };

    my $ShippingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);
    return $ShippingAddress;
}

sub _createShopAddress {
    my $self = shift;

    my $hAttributes = {
        'Company' => 'ShopTest Company',
        'FullName' => 'ShopTest FullName',
        'Street' => 'ShopTest street',
        'Street2' => 'ShopTest street2',
        'Zipcode' => '50000',
        'City' => 'ShopTestTown',
        'State' => 'ShopTest State',
        'Country' => {'Code2' => 'US'},
    };

    my $ShopAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);
    return $ShopAddress;
}

sub _createBillingAddress {
    my $self = shift;
    my ($VATID) = @_;

    my $hAttributes = {
        'Company' => 'CustomerTest Company',
        'FullName' => 'CustomerTest FullName',
        'Street' => 'CustomerTest street',
        'Street2' => 'CustomerTest street2',
        'Zipcode' => '50000',
        'City' => 'CustomerTestTown',
        'State' => 'CustomerTest State',
        'Country' => {'Code2' => 'US'},
        'VATID' => $VATID,
    };

    my $BillingAddress = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);
    return $BillingAddress;
}

sub _createLineItemAvaTax {
    my $self = shift;
    my $hAttributes = {
        'instanceOf' => 'LineItemAvaTax',
        'GUID' => 'LineItemAvaTax GUID',
        'Name' => 'LineItemAvaTax',
    };
    my $MockedLineItemAvaTax = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::AvaTax::API::Object::LineItemAvaTax', $hAttributes);
    return $MockedLineItemAvaTax;
}

sub _createBasketForAvaTax {
    my $self = shift;
    my $LineItemAvaTax = $self->_createLineItemAvaTax();
    my $LineItemContainer = $self->_createLineItemContainer();
    $LineItemContainer->set({'Children' => [$LineItemAvaTax]});
    $LineItemContainer->set({'AvaTax_Checksum' => 123});
    return $self->_createBasket($LineItemContainer);
}

sub _createOrder {
    my $self = shift;
    my ($LineItemContainer, $Customer) = @_;
    my $hAttributes = {
        'Alias' => 'MyOrder',
        'instanceOf' => 'CustomerOrder',
        'Customer' => $Customer,
        'Site' => $self->_createShop($self->_createTaxMethodAvaTax()),
        'LineItemContainer' => $LineItemContainer,
        'ShippingAddress' => $self->_createShippingAddress(),
        'ShopAddress' => $self->_createShopAddress(),
        'PaidOn' => undef,
    };
    my $MockOrder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes);
    $MockOrder->mock('class', sub {
        return  $self->_createClass();
    });
    return $MockOrder;
}

sub _getTestVATID {
    return 666666;
}

#--------------------------------------------------------------------------------------
# Expected error message
#--------------------------------------------------------------------------------------
sub _expectedErrorMissingShopAddress {
    my $self = shift;
    return {
          'Code' => 'MissingAddressInfo',
          'Info' => {
                      'Street2' => 'ShopTest street2',
                      'Country' => {
                                     'Code2' => 'US'
                                   },
                      'Zipcode' => undef,
                      'Street' => 'ShopTest street',
                      'City' => 'ShopTestTown',
                      'State' => 'ShopTest State',
                      'FullName' => 'ShopTest FullName',
                      'Company' => 'ShopTest Company'
                    }
        };
}

sub _expectedErrorMissingCustomerAddress {
    my $self = shift;
    return {
          'Code' => 'MissingAddressInfo',
          'Info' => {
                'Company' => 'CustomerTest Company',
                'FullName' => 'CustomerTest FullName',
                'Street' => 'CustomerTest street',
                'Street2' => 'CustomerTest street2',
                'Zipcode' => 'Test Zipcode',
                'City' => 'CustomerTestTown',
                'State' => 'CustomerTest State',
                'Country' => {'Code2' => 'Not US'},
                    }
        };
}


sub _expectedLineItemContainerAttributes {
    my $self = shift;
    return {
          'LanguageID' => 'ID',
          'CurrencyID' => 'USD',
          'TaxModel' => 0
        };
}
sub _expectedShopAddressObject {
    my $self = shift;
    my $ExpectedShopAddressObject = bless( {
                 'Line2' => 'ShopTest street',
                 'PostalCode' => '50000',
                 'AddressCode' => 'Origin',
                 'City' => 'ShopTestTown',
                 'Region' => 'ShopTest State',
                 'Line3' => 'ShopTest street2',
                 'Line1' => 'ShopTest Company',
                 'Country' => 'US',
               }, 'DE_EPAGES::AvaTax::API::DTO::BaseAddress' );
    return $ExpectedShopAddressObject;
}

sub _expectedCustomerAddressObject {
    my $self = shift;
    my $ExpectedShopAddressObject = bless( {
                 'Line2' => 'CustomerTest street',
                 'PostalCode' => '50000',
                 'AddressCode' => 'Dest',
                 'City' => 'CustomerTestTown',
                 'Region' => 'CustomerTest State',
                 'Line3' => 'CustomerTest street2',
                 'Line1' => 'CustomerTest Company',
                 'Country' => 'US',
               }, 'DE_EPAGES::AvaTax::API::DTO::BaseAddress' );
    return $ExpectedShopAddressObject;
}

## use critic (ProhibitLongLines)
__PACKAGE__->runScript();