#========================================================================================
# package      DE_EPAGES::PayPalPlus::t::unit::ExecutePayment_Repository
# base         DE_EPAGES::Core::API::Test
#----------------------------------------------------------------------------------------
# description  unit test for ExecutePayment::Repository
#========================================================================================
package DE_EPAGES::PayPalPlus::t::unit::ExecutePayment_Repository;
use base DE_EPAGES::Core::API::Test;
use Test::More;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Test::Mock::Mockify;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use strict;

sub test {
    my $self = shift;

    $self->test_BuildRequest_Successful();
    $self->test_BuildRequest_MissingPayerID();

    $self->test_WriteResponse_Pending();
    $self->test_WriteResponse_Paid();
    $self->test_WriteResponse_PartlyPaid();
    $self->test_WriteResponse_OrderAlreadyPaid();
    $self->test_WriteResponse_WithBasket();

    $self->test_VerifyResponse_Successful();
    $self->test_VerifyResponse_MissingTransactionID();
    $self->test_VerifyResponse_UnexpectedAmountOf_Transactions();
    $self->test_VerifyResponse_UnexpectedAmountOf_RelatedResources();
    $self->test_VerifyResponse_BrokenResponse();

    $self->test_VerifyResponse_PUI_Successful();
    $self->test_VerifyResponse_PUI_UnknownInstructionType();
    $self->test_VerifyResponse_PUI_MissingIBAN();
    $self->test_VerifyResponse_PUI_DifferentDateFormat();

    done_testing();
}

sub test_BuildRequest_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder();

    $Repository->buildRequest($CustomerOrder);
    my $RequestDTO = $Repository->getRequestDTO();
    my $PayerId = $RequestDTO->payerId();

    is($PayerId, 'someID', "$SubTestName - Test if PayerID is right");
}

sub test_BuildRequest_MissingPayerID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder_MissingPayerID();

    throws_ok( sub { $Repository->buildRequest($CustomerOrder); }, qr/PARAMETER_MISSING: Missing parameter PayerID/, "$SubTestName - Test if buildRequest dies");
}

sub test_WriteResponse_Pending {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder();

    my $hResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'pending',
        'Amount' => '70.65',
        'Currency' => 'EUR',
    };

    $Repository->writeResponse($hResponse, $CustomerOrder);

    my $LineItemPaymentPayPalPlus = $CustomerOrder->container()->get('Payment');
    my $TransStatus = $LineItemPaymentPayPalPlus->get('TransStatus');
    my $TransID = $LineItemPaymentPayPalPlus->get('TransID');
    my $Amount = $LineItemPaymentPayPalPlus->get('Amount');
    my $Currency = $LineItemPaymentPayPalPlus->get('CurrencyID');

    is($TransStatus, 'STATUS_PENDING', "$SubTestName - Test if TransStatus is STATUS_PENDING");
    is($TransID, 'TransactionIdFromPaypal', "$SubTestName - Test if TransID is TransactionIdFromPaypal");
    is($Amount, '70.65', "$SubTestName - Test if Amount was set correctly");
    is($Currency, 'EUR', "$SubTestName - Test if Currency was set correctly");
}

sub test_WriteResponse_Paid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder();

    my $hResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '70.65',
        'Currency' => 'EUR',
    };

    $Repository->writeResponse($hResponse, $CustomerOrder);

    my $LineItemPaymentPayPalPlus = $CustomerOrder->container()->get('Payment');
    my $PaidOn = $CustomerOrder->get('PaidOn');
    my $TransStatus = $LineItemPaymentPayPalPlus->get('TransStatus');
    my $TransTime = $LineItemPaymentPayPalPlus->get('TransTime');
    my $TransID = $LineItemPaymentPayPalPlus->get('TransID');
    my $Amount = $LineItemPaymentPayPalPlus->get('Amount');
    my $Currency = $LineItemPaymentPayPalPlus->get('CurrencyID');

    is($TransStatus, 'STATUS_PAID', "$SubTestName - Test if TransStatus is STATUS_PAID");
    is($TransTime, 'now', "$SubTestName - Test if TransTime is now");
    is($PaidOn, 'now', "$SubTestName - Test if PaidOn is now");
    is($TransID, 'TransactionIdFromPaypal', "$SubTestName - Test if TransID is TransactionIdFromPaypal");
    is($Amount, '70.65', "$SubTestName - Test if Amount was set correctly");
    is($Currency, 'EUR', "$SubTestName - Test if Currency was set correctly");
}

sub test_WriteResponse_PartlyPaid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder();

    my $hResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '2',
        'Currency' => 'USD',
    };

    $Repository->writeResponse($hResponse, $CustomerOrder);

    my $LineItemPaymentPayPalPlus = $CustomerOrder->container()->get('Payment');
    my $PartlyPaidOn = $CustomerOrder->get('PartlyPaidOn');
    my $PaidOn = $CustomerOrder->get('PaidOn');
    my $TransStatus = $LineItemPaymentPayPalPlus->get('TransStatus');
    my $TransTime = $LineItemPaymentPayPalPlus->get('TransTime');
    my $TransID = $LineItemPaymentPayPalPlus->get('TransID');
    my $Amount = $LineItemPaymentPayPalPlus->get('Amount');
    my $Currency = $LineItemPaymentPayPalPlus->get('CurrencyID');

    is($TransStatus, undef, "$SubTestName - Test if TransStatus is STATUS_PAID");
    is($TransTime, undef, "$SubTestName - Test if TransTime is now");
    is($PartlyPaidOn, 'now', "$SubTestName - Test if PartlyPaidOn is now");
    is($PaidOn, undef, "$SubTestName - Test if PaidOn is not ste");
    is($TransID, 'TransactionIdFromPaypal', "$SubTestName - Test if TransID is TransactionIdFromPaypal");
    is($Amount, '2', "$SubTestName - Test if Amount was set correctly");
    is($Currency, 'USD', "$SubTestName - Test if Currency was set correctly");
}

sub test_WriteResponse_OrderAlreadyPaid {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $CustomerOrder = $self->_createCustomerOrder_AlreadyPaid();
    my $hResponse = {};

    throws_ok( sub { $Repository->writeResponse($hResponse, $CustomerOrder); }, qr/OrderAlreadyPaid/, "$SubTestName - Test if writeResponse dies");
}

sub test_WriteResponse_WithBasket {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $Basket = $self->_createBasket();
    my $hResponse = {};

    throws_ok( sub { $Repository->writeResponse($hResponse, $Basket); }, qr/WRONG_CLASS: CustomerOrder/, "$SubTestName - Test if writeResponse dies");
}

sub test_VerifyResponse_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = $self->_buildResponse({
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
    });

    my $hVerifiedResponse;
    lives_ok( sub { $hVerifiedResponse = $Repository->verifyResponse($hResponse); }, "$SubTestName - Test if writeResponse lives");

    my $hExpectedResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'ReferenceNumber' => undef,
            'PaymentDueDate' => undef,
            'BankName' => undef,
            'AccountHolderName' => undef,
            'IBAN' => undef,
            'BIC' => undef,
            'Amount' => undef,
            'Currency' => undef,
        },
    };

    is_deeply($hVerifiedResponse, $hExpectedResponse, "$SubTestName - Test if verified response is correct");
}

sub test_VerifyResponse_PUI_UnknownInstructionType {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = $self->_buildInvoiceResponse({
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'InstructionType' => 'unknown',
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '2015-09-24',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => 'DE07120700888000XXXXXX',
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    });

    my $hVerifiedResponse;
    lives_ok( sub { $hVerifiedResponse = $Repository->verifyResponse($hResponse); }, "$SubTestName - Test if verifyResponse lives");

    my $hExpectedResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'ReferenceNumber' => undef,
            'PaymentDueDate' => undef,
            'BankName' => undef,
            'AccountHolderName' => undef,
            'IBAN' => undef,
            'BIC' => undef,
            'Amount' => undef,
            'Currency' => undef,
        },
    };

    is_deeply($hVerifiedResponse, $hExpectedResponse, "$SubTestName - Test if verified response is correct");
}

sub test_VerifyResponse_PUI_DifferentDateFormat {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $DateTimeFormatter = $self->_createDateTimeFormatter_Error();
    my $Repository = $self->_createRepository($DateTimeFormatter);
    my $hResponse = $self->_buildInvoiceResponse({
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'InstructionType' => 'PAY_UPON_INVOICE',
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '24.09.2015',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => 'DE07120700888000XXXXXX',
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    });

    my $hVerifiedResponse;
    lives_ok( sub { $hVerifiedResponse = $Repository->verifyResponse($hResponse); }, "$SubTestName - Test if verifyResponse lives");

    my $hExpectedResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '24.09.2015',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => 'DE07120700888000XXXXXX',
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    };

    is_deeply($hVerifiedResponse, $hExpectedResponse, "$SubTestName - Test if verified response is correct");
}

sub test_VerifyResponse_PUI_Successful {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = $self->_buildInvoiceResponse({
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'InstructionType' => 'PAY_UPON_INVOICE',
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '2015-09-24',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => 'DE07120700888000XXXXXX',
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    });

    my $hVerifiedResponse;
    lives_ok( sub { $hVerifiedResponse = $Repository->verifyResponse($hResponse); }, "$SubTestName - Test if verifyResponse lives");

    my $hExpectedResponse = {
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'Amount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '2015-09-24T00:00:00',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => 'DE07120700888000XXXXXX',
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    };

    is_deeply($hVerifiedResponse, $hExpectedResponse, "$SubTestName - Test if verified response is correct");
}

sub test_VerifyResponse_PUI_MissingIBAN {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = $self->_buildInvoiceResponse({
        'TransactionId' => 'TransactionIdFromPaypal',
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
        'InvoiceData' => {
            'InstructionType' => 'PAY_UPON_INVOICE',
            'ReferenceNumber' => '7FC35911MD7697828',
            'PaymentDueDate' => '2015-09-24',
            'BankName' => 'Demo Bank',
            'AccountHolderName' => 'Demo Company GmbH',
            'IBAN' => undef,
            'BIC' => 'DEMTDEDBPAL',
            'Amount' => '72.00',
            'Currency' => 'EUR',
        },
    });

    throws_ok( sub { $Repository->verifyResponse($hResponse); }, qr/Missing parameter IBAN/, "$SubTestName - Test if verifyResponse dies");
}

sub test_VerifyResponse_MissingTransactionID {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = $self->_buildResponse({
        'State' => 'completed',
        'TotalAmount' => '70.65',
        'Currency' => 'EUR',
    });

    throws_ok( sub { $Repository->verifyResponse($hResponse); }, qr/Missing parameter TransactionId/, "$SubTestName - Test if writeResponse dies");
}

sub test_VerifyResponse_UnexpectedAmountOf_Transactions {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = {
        'transactions' => [
            'first',
            'last',
        ]
    };

    throws_ok( sub { $Repository->verifyResponse($hResponse); },
               qr/UNSUPPORTED_ARRAY_SIZE: PayPal plus only support one element for Transactions, actual size: 2/,
               "$SubTestName - Test if writeResponse dies when to many Transactions are returned"
    );
}

sub test_VerifyResponse_UnexpectedAmountOf_RelatedResources {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = {
        'transactions' => [
            {'related_resources'=>[
                'first',
                'last',
                ]
            }
        ]
    };

    throws_ok( sub { $Repository->verifyResponse($hResponse); },
               qr/UNSUPPORTED_ARRAY_SIZE: PayPal plus only support one element for related_resources, actual size: 2/,
               "$SubTestName - Test if writeResponse dies when to many Transactions are returned"
    );
}

sub test_VerifyResponse_BrokenResponse {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $Repository = $self->_createRepository();
    my $hResponse = {'transactions'=>'this is not a transaction'};

    throws_ok( sub { $Repository->verifyResponse($hResponse); }, qr/PARAMETER_NO_ARRAY/, "$SubTestName - Test if writeResponse dies");
}

#----------------------------------------------------------------------------------------
# helpers
#----------------------------------------------------------------------------------------

sub _buildResponse {
    my $self = shift;
    my ($hData) = @_;

    my $hResponse = {
        'id' => $hData->{'TransactionId'},
        'state' => $hData->{'State'},
        'transactions' => [{
            'related_resources' => [{
                'sale' => {
                    'id' => $hData->{'TransactionId'},
                    'state' => $hData->{'State'},
                },
            }],
            'amount' => {
                'total' => $hData->{'TotalAmount'},
                'currency' => $hData->{'Currency'},
            },
        }],
    };
    return $hResponse;
}
#----------------------------------------------------------------------------------------
sub _buildInvoiceResponse {
    my $self = shift;
    my ($hData) = @_;

    my $hResponse = $self->_buildResponse($hData);
    my $InvoiceData = $hData->{'InvoiceData'};

    $hResponse->{'payment_instruction'} = {
        'reference_number' => $InvoiceData->{'ReferenceNumber'},
        'instruction_type' => $InvoiceData->{'InstructionType'},
        'payment_due_date' => $InvoiceData->{'PaymentDueDate'},
        'recipient_banking_instruction' => {
            'bank_name' => $InvoiceData->{'BankName'},
            'account_holder_name' => $InvoiceData->{'AccountHolderName'},
            'international_bank_account_number' => $InvoiceData->{'IBAN'},
            'bank_identifier_code' => $InvoiceData->{'BIC'},
        },
        'amount' => {
            'value' => $InvoiceData->{'Amount'},
            'currency' => $InvoiceData->{'Currency'},
        }
    };

    return $hResponse;
}

#----------------------------------------------------------------------------------------
sub _createRepository {
    my $self = shift;
    my ($DateTimeFormatter) = @_;
    $DateTimeFormatter //= $self->_createDateTimeFormatter();

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::Repository', []);
    $Mockify->addMockWithReturnValue('_GetCurrentDateTime', 'now');
    $Mockify->addMockWithReturnValue('_createRequestDTO', $self->_createRequestDTO());
    $Mockify->addMockWithReturnValue('_createDateTimeFormatter', $DateTimeFormatter);
    $Mockify->addMockWithReturnValueAndParameterCheck('_logError', '', [
        {'object' => 'DE_EPAGES::Core::API::Error'}
    ]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createRequestDTO {
    my $self = shift;
    return DE_EPAGES::PayPalPlus::API::Webservice::ExecutePayment::RequestDTO->new();
}
#----------------------------------------------------------------------------------------
sub _createDateTimeFormatter {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Core::API::Object::DateTimeFormatter', []);
    $Mockify->addMockWithReturnValueAndParameterCheck('parse_datetime', '2015-09-24T00:00:00', [
        {'string' => '2015-09-24'},
        'string',
        'string',
    ]);
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createDateTimeFormatter_Error {
    my $self = shift;

    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::Core::API::Object::DateTimeFormatter', []);
    $Mockify->addMock('parse_datetime', sub { Error('WrongDateFormat'); });
    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createBasket {
    my $self = shift;
    my $Basket = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::Basket', {} );
    return $Basket;
}
#----------------------------------------------------------------------------------------
sub _createCustomerOrder {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer(),
        'PaidOn' => undef
    };
    my $CustomerOrder = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes );

    return $CustomerOrder;
}
#----------------------------------------------------------------------------------------
sub _createCustomerOrder_MissingPayerID {
    my $self = shift;

    my $hAttributes = {
        'LineItemContainer' => $self->_createLineItemContainer_MissingPayerID(),
        'PaidOn' => undef
    };
    my $CustomerOrder = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes );

    return $CustomerOrder;
}
#----------------------------------------------------------------------------------------
sub _createCustomerOrder_AlreadyPaid {
    my $self = shift;

    my $hAttributes = {
        'PaidOn' => 'a long time ago'
    };
    my $CustomerOrder = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes );

    return $CustomerOrder;
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createLineItemPayment(),
        'GrandTotal' => '70.65',
        'CurrencyID' => 'EUR',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );

    return $LineItemContainer;
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer_MissingPayerID {
    my $self = shift;

    my $hAttributes = {
        'Payment' => $self->_createLineItemPayment_MissingPayerID(),
        'GrandTotal' => '70.65',
        'CurrencyID' => 'EUR',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes );

    return $LineItemContainer;
}
#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my $self = shift;

    my $hAttributes = {
        'PayerID' => 'someID',
        'TransID2' => 'Dummmy_PaymentId',
        'Amount' => undef,
        'TransStatus' => undef,
        'TransTime' => undef,
        'PaymentMethod' => $self->_createPaymentMethodPayPalPlus(),
    };
    my $LineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus', $hAttributes );

    return $LineItemPayment;
}
#----------------------------------------------------------------------------------------
sub _createLineItemPayment_MissingPayerID {
    my $self = shift;

    my $hAttributes = {
        'PayerID' => undef,
        'PaymentMethod' => $self->_createPaymentMethodPayPalPlus(),
    };
    my $LineItemPayment = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::LineItemPaymentPayPalPlus', $hAttributes );

    return $LineItemPayment;
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethodPayPalPlus {
    my $self = shift;

    my $hAttributes = {
        'IsInTestMode' => 1,
        'PaymentType' => $self->_createPaymentTypePayPalPlus(),
        'MerchantClientId' => 'Dummy_MerchantClientId',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::PayPalPlus::API::Object::PaymentMethodPayPalPlus', $hAttributes );
}
#----------------------------------------------------------------------------------------
sub _createPaymentTypePayPalPlus {
    my $self = shift;

    my $hAttributes = {
        'ProviderClientId' => 'Dummy_ProviderClientId',
        'ProviderClientSecret' => 'Dummy_ProviderClientSecret',
        'PayPalPartnerAttributionId' => 'MyAttributionId',
    };
    my $PaymentTypePayPalPlus = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Payment::API::Object::PaymentType', $hAttributes );

    return $PaymentTypePayPalPlus;
}

__PACKAGE__->runScript();
